@@ -37,7 +37,7 @@ def test(
 
         with ExitTrap(destroy):
             # capture & print logs
-            logs = cluster.logs(task)
+            logs = cluster.logs(task.id)
             logger.header('task output')
             for msg in logs:
                 logger.handle(msg)
