@@ -1,4 +1,5 @@
-import aiohttp
+from aiohttp import WSCloseCode
+from datetime import datetime
 from .rpc_client import RpcClient
 
 
@@ -10,11 +11,15 @@ class Conn(object):
         self.remote = remote
 
     async def send(self, msg: dict) -> None:
+        msg['ts'] = datetime.now().isoformat()
+        if 'type' not in msg:
+            raise Exception('Messages must have a type field')
+
         await self.ws.send_json(msg)
 
     async def close(self, message=''):
         self.rpc.cancel_all()
         await self.ws.close(
-            code=aiohttp.WSCloseCode.GOING_AWAY,
+            code=WSCloseCode.GOING_AWAY,
             message=message
         )
