@@ -1,8 +1,8 @@
 from datetime import datetime, timezone
 from cowait.tasks import Task, TaskDefinition, sleep, rpc
-from cowait.network import Conn, get_local_connstr
-from cowait.tasks.status import DONE, FAIL
-from cowait.tasks.messages import TASK_INIT, TASK_STATUS, TASK_FAIL
+from cowait.network import Conn, get_local_url
+from cowait.tasks.status import WAIT, WORK, STOP
+from cowait.tasks.messages import TASK_INIT, TASK_STATUS
 from .tasklist import TaskList
 from .subscriptions import Subscriptions
 from .api import Dashboard, TaskAPI
@@ -25,15 +25,18 @@ class Agent(Task):
         self.subs.on('subscribe', send_state)
 
         self.token = self.meta['http_token']
+        if self.token is None or self.token == '':
+            self.node.server.auth.enabled = False
+            self.token = 'none'
 
         # create http server
-        self.node.http.add_routes(TaskAPI(self).routes('/api/1/tasks'))
-        self.node.http.add_routes(Dashboard().routes())
-        self.node.http.auth.add_token(self.token)
+        self.node.server.add_routes(TaskAPI(self).routes('/api/1/tasks'))
+        self.node.server.add_routes(Dashboard().routes())
+        self.node.server.auth.add_token(self.token)
 
     async def run(self, **inputs) -> dict:
-        if '/' in self.routes:
-            url = self.routes['/']['url']
+        if '/' in self.taskdef.routes:
+            url = self.taskdef.routes['/']['url']
             print('Agent ready. Dashboard available at:')
             print(f'{url}?token={self.token}')
         else:
@@ -43,8 +46,24 @@ class Agent(Task):
         while True:
             running_tasks = self.cluster.list_all()
             for id, task in self.tasks.items():
-                # skip tasks that are done or failed
-                if task.status == DONE or task.status == FAIL:
+                # consider only tasks that are waiting or running
+                if task.status != WAIT and task.status != WORK:
+                    continue
+
+                # check if parent is dead
+                if task.parent is not None and task.parent not in running_tasks:
+                    parent = self.tasks[task.parent]
+                    virtual_parent = parent.meta.get('virtual', False)
+                    if not virtual_parent or parent.status != WORK:
+                        since = datetime.now(timezone.utc) - task.created_at
+                        await self.subtasks.emit_child_error(id, f'Task lost parent after {since}')
+
+                        # destroy it
+                        self.cluster.destroy(task.id)
+
+                # special case for virtual tasks: since they dont exist as containers,
+                # we cant check if they are still alive.
+                if task.meta.get('virtual', False):
                     continue
 
                 # ensure task is still in the list of running tasks
@@ -53,18 +72,7 @@ class Agent(Task):
                     # compute task age
                     since = datetime.now(timezone.utc) - task.created_at
                     error = f'Task lost after {since}'
-
-                    # update task state record
-                    await self.tasks.on_status(None, id=id, status=FAIL)
-                    await self.tasks.on_fail(None, id=id, error=error)
-
-                    # emulate failure messages to parent
-                    await self.node.parent.msg(type=TASK_STATUS, id=id, status=FAIL)
-                    await self.node.parent.msg(type=TASK_FAIL, id=id, error=error)
-
-                    # emulate failure messages to subscribers
-                    await self.subs.forward(None, id=id, type=TASK_FAIL, error=error)
-                    await self.subs.forward(None, id=id, type=TASK_STATUS, status=FAIL)
+                    await self.subtasks.emit_child_error(id, error)
 
             await sleep(5.0)
 
@@ -72,10 +80,14 @@ class Agent(Task):
 
     @rpc
     async def destroy(self, task_id) -> None:
+        await self.emulate_stop(task_id)
         self.cluster.destroy(task_id)
 
     @rpc
     async def destroy_all(self) -> None:
+        for id, task in self.tasks.items():
+            if task.status == WAIT or task.status == WORK:
+                await self.emulate_stop(id)
         self.cluster.destroy_all()
 
     @rpc
@@ -84,7 +96,7 @@ class Agent(Task):
 
     @rpc
     async def get_agent_url(self) -> str:
-        url = get_local_connstr()
+        url = get_local_url()
         return f'{url}?token={self.token}'
 
     @rpc
@@ -112,7 +124,7 @@ class Agent(Task):
             id=id,
             name=name,
             image=image,
-            upstream=get_local_connstr(),
+            upstream=get_local_url(),
             meta=meta,
             ports=ports,
             routes=routes,
@@ -127,9 +139,12 @@ class Agent(Task):
         ))
 
         # authorize id
-        self.node.http.auth.add_token(id)
+        self.node.server.auth.add_token(id)
 
         # register with subtask manager
         self.subtasks.watch(task)
 
         return task.serialize()
+
+    async def emulate_stop(self, task_id: str):
+        await self.node.server.emit(type=TASK_STATUS, id=task_id, status=STOP, conn=None)
